C--------------------------------------------------------------------
C  Copyright (C) 2002,  Space Science and Engineering Center,
C  University C  of Wisconsin-Madison, Madison WI.
C
C  This program is free software; you can redistribute it
C  and/or modify it under the terms of the GNU General
C  Public License as published by the Free Software Foundation;
C  either version 2 of the License, or (at your option) any
C  later version.
C
C  This program is distributed in the hope that it will be
C  useful, but WITHOUT ANY WARRANTY; without even the implied
C  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
C  See the  GNU General Public License for more details.
C
C  You should have received a copy of the GNU General Public
C  License along with this program; if not, write to the Free
C  Software Foundation, Inc., 59 Temple Place, Suite 330,
C  Boston, MA  02111-1307 USA
C--------------------------------------------------------------------
C
C
      SUBROUTINE DB_GET_SFCPRES_PWAT(ANC_MET_LUN,LAT,LON,SFCPRES,GDAS_PWAT)
      
C-----------------------------------------------------------------------
C!F77
C
C!DESCRIPTION:
C     Gets bilinear interpolated surface pressure for a given lat/lon.
C
C!INPUT PARAMETERS:
C     ANC_MET_LUN Unit Number GDAS ancillary data file
C     LAT         Latitude (degrees, -90S to +90N)
C     LON         Longitude (degrees, -180W to +180E, Greenwich zero)
C
C!OUTPUT PARAMETERS:
C     SFCPRES    Interpolated surface pressure (hPa)
C     GDAS_PWAT  Interpolated precipitable water (mm)
C
C!REVISION HISTORY:
C     Developed by the MODIS Group, CIMSS/SSEC, UW-Madison.
C March 03
C     SWS Modified get_sfcpres.f to read PWAT
C
C!END
C-----------------------------------------------------------------------

      implicit none
      
c ... Input arguments     
      real lat, lon
      
c ... Output arguments
      real sfcpres
      real gdas_pwat
      
c ... Local variables
      real pres(0:15), temp(0:15), mixr(0:15),
     &  land, sfctmp, ugrd, vgrd, ozone, icec, sst
      integer nise, status, anc_met_lun, lun_icec, lun_ozone,
     &        lun_sst, lun_nisen, lun_nises
      real lat1, lat2, lon1, lon2
      real pwat1, pwat2, pwat3, pwat4
      real p1, p2, p3, p4
      real t, u

c ... External functions
      integer floor, ceil, db_get_ancillary
      external floor, ceil, db_get_ancillary

c ... Set lun numbers needed by get_ancillary routine.  
      lun_icec = 0
      lun_ozone = 0
      lun_sst = 0
      lun_nisen = 0
      lun_nises = 0

            
c ... Set default return value
      sfcpres = -1.0

c ... Get coordinates of grid points surrounding this lat/lon
      lat1 = real(floor(lat))
      lat2 = real(ceil(lat))
      if (abs(lat2 - lat1) .lt. 1.0e-6) lat2 = lat1 + 1.0
      lon1 = real(floor(lon))
      lon2 = real(ceil(lon))
      if (abs(lon2 - lon1) .lt. 1.0e-6) lon2 = lon1 + 1.0

c ... Get the surface pressures at the 4 grid points
c ... surrounding this lat/lon
      status = db_get_ancillary(anc_met_lun, lun_ozone, lun_icec,
     &  lun_sst, lun_nisen, lun_nises, lat1, lon1, pres, temp, mixr, 
     &  land, sfctmp, p1, pwat1, ugrd, vgrd, ozone, icec, sst, nise)
      if ((p1 .lt. 10.0) .or. (p1 .gt. 1200.0)) return
      if ((pwat1 .lt. 0.0) .or. (pwat1 .gt. 110.0)) return
      
      status = db_get_ancillary(anc_met_lun, lun_ozone, lun_icec,
     &  lun_sst, lun_nisen, lun_nises, lat1, lon2, pres, temp, mixr, 
     &  land, sfctmp, p2, pwat2, ugrd, vgrd, ozone, icec, sst, nise)
      if ((p2 .lt. 10.0) .or. (p2 .gt. 1200.0)) return
      if ((pwat2 .lt. 0.0) .or. (pwat2 .gt. 110.0)) return

      status = db_get_ancillary(anc_met_lun, lun_ozone, lun_icec,
     &  lun_sst, lun_nisen, lun_nises, lat2, lon2, pres, temp, mixr, 
     &  land, sfctmp, p3, pwat3, ugrd, vgrd, ozone, icec, sst, nise)
      if ((p3 .lt. 10.0) .or. (p3 .gt. 1200.0)) return
      if ((pwat3 .lt. 0.0) .or. (pwat3 .gt. 110.0)) return
     
      status = db_get_ancillary(anc_met_lun, lun_ozone, lun_icec,
     &  lun_sst, lun_nisen, lun_nises, lat2, lon1, pres, temp, mixr, 
     &  land, sfctmp, p4, pwat4, ugrd, vgrd, ozone, icec, sst, nise)
      if ((p4 .lt. 10.0) .or. (p4 .gt. 1200.0)) return
      if ((pwat4 .lt. 0.0) .or. (pwat4 .gt. 110.0)) return

c ... Compute bilinear interpolated surface pressure
      u = (lat - lat1) / (lat2 - lat1)
      t = (lon - lon1) / (lon2 - lon1)
      sfcpres =
     &  (1.0 - t) * (1.0 - u) * p1 +
     &  t * (1.0 - u) * p2 +
     &  t * u * p3 +
     &  (1.0 - t) * u * p4

      gdas_pwat =
     &  (1.0 - t) * (1.0 - u) * pwat1 +
     &  t * (1.0 - u) * pwat2 +
     &  t * u * pwat3 +
     &  (1.0 - t) * u * pwat4

      END
