C--------------------------------------------------------------------
C  Copyright (C) 2002,  Space Science and Engineering Center, 
C  University C  of Wisconsin-Madison, Madison WI.
C      
C  This program is free software; you can redistribute it 
C  and/or modify it under the terms of the GNU General 
C  Public License as published by the Free Software Foundation; 
C  either version 2 of the License, or (at your option) any 
C  later version.
C
C  This program is distributed in the hope that it will be 
C  useful, but WITHOUT ANY WARRANTY; without even the implied 
C  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
C  See the  GNU General Public License for more details.
C
C  You should have received a copy of the GNU General Public 
C  License along with this program; if not, write to the Free 
C  Software Foundation, Inc., 59 Temple Place, Suite 330, 
C  Boston, MA  02111-1307 USA
C--------------------------------------------------------------------
C
C
      subroutine db_mod07_get_data(l1b_1km_lun,geo_1km_lun,datatype_1km,
     &           datatype_geo,datat_mask,interleave_1km,interleave_geo,
     &           interl_mask,resolution_1km,resolution_geo,resol_mask,
     &           offset_1km,offset_geo,offset_mask,samples_1km,
     &           samples_geo,sampl_mask,lines_1km,lines_geo,lines_mask,
     &           error_1km,error_geo,bands_1km,bands_geo,bands_mask,
     &           bandnames_1km,bandnames_geo,bnames_mask,bandunits_1km,
     &           bandunits_geo,scan_1km_lun,anc_met_lun,mask_lun,
     &           scan)

c-----------------------------------------------------------------------
c
c!F77
c
c!Description:
c    Get all input data at 1km resolution needed by the University
c    of Wisconsin MOD07 Atmospheric Profiles Product.
c
c!Input Parameters:
C    l1b_1km_lun     LUN for open 1km L1B data file
C    geo_1km_lun     LUN for open geolocation 1km file
C    datatype_1km    Format of data in L1b file
C    datatype_geo    Format of data in geolocation file
C    datat_mask      Format of data in cloud mask file
C    interleave_1km  Order data is saved in L1b file
C    interleave_geo  Order data is saved in geolocation file
C    interl_mask     Order data is saved in cloud mask file
C    resolution_1km  L1b file data resolution
C    resolution_geo  Geolocation file data resolution
C    resol_mask      Cloud mask file data resolution
C    offset_1km      Offset, if any, of 1km L1B data
C    offset_geo      Offset, if any, of geolocation file data
C    offset_mask     Offset, if any, of cloud mask file data
C    samples_1km     Number of elements in 1km L1b file
C    samples_geo     Number of elements in geolocation file
C    sampl_mask      Number of elements in cloud mask file
C    lines_1km       Number of lines of data in 1km L1b file
C    lines_geo       Number of lines of data in geolocation file
C    lines_mask      Number of lines of data in cloud mask file
C    error_1km       Bad data value of data in 1km L1b file
C    error_geo       Bad data value of data in geolocation file
C    bands_1km       Number of bands in 1km L1b file
C    bands_geo       Number of bands in geolocation file
C    bands_mask      Number of bands in cloud mask file
C    bandnames_1km   Band numbers in 1km L1b file
C    bandnames_geo   Band numbers in geolocation file
C    bnames_mask     Band numbers in cloud mask file
C    bandunits_1km   Units for each band in the 1km L1b data file
C    bandunits_geo   Units for each band in the geolocation file
C    scan_1km_lun    LUN for scan based metadata file
C    anc_met_lun     LUN for open met data file
C    mask_lun        LUN for open Cloud Mask file
c    scan            Scan number within L1B granule
c
c!Output Parameters:
c     The following arrays in COMMON /MOD07_DATA/ are filled:
c     RADIANCE1      Radiances for IR bands
c     LAT1           Latitude
c     LON1           Longitude
c     ELV1           Surface elevation
c     ZEN1           Sensor zenith angle
c     TEM1           Surface temperature
c     WAT1           Surface water vapor mixing ratio
c     PRE1           Surface pressure
c     LAND1          Land/water mask
c     MASK1          Cloudmask
c     PWAT1          GDAS Total Precipitable Water
c
c!Revision History:
c     March 2003: SWS added PWAT1
c
c!Team-unique Header:
c     Developed by the MODIS Group, CIMSS/SSEC, UW-Madison.
c
c!End
c
c-----------------------------------------------------------------------

      implicit none
      
      include 'db_mod07uw_data.inc'
      include 'db_mod07uw_debug.inc'

c ... arguments

      integer l1b_1km_lun,geo_1km_lun,scan_1km_lun,
     +        mask_lun,scan,datat_mask,datatype_1km,datatype_geo,
     +        resolution_1km,resolution_geo,resol_mask,offset_1km,
     +        offset_geo,offset_mask,samples_1km,samples_geo,sampl_mask,
     +        lines_1km,lines_geo,lines_mask,bands_1km,bands_geo,
     +        bands_mask,anc_met_lun

      real error_1km,error_geo,error_mask
 
      character*(*) interleave_1km,bandnames_1km(*),bandunits_1km(*),
     +              interleave_geo,bandnames_geo(*),bandunits_geo(*),
     +              interl_mask,bnames_mask(*)

c ... local variables

      integer nise, resol, data_size( 2 )
      character*80 bunits_mask(NMASK)
      character*6 cal_type,band_no
      character*80 req_bandname
      
      real buffer( max_pixel *  max_line )
      byte v_flag( max_pixel *  max_line ), 
     &        m_buf (max_pixel *  max_line)

      integer i, j, k, m, n, mask_byte, mirror_side, 
     &        scan_flag, status, lun_ozone, lun_icec,
     &        lun_sst, lun_nisen, lun_nises

      real pres( 16 ), temp( 16 ), mixr( 16 ), land, sfctmp, sfcpres ,
     &  pwat, ugrd, vgrd, ozone, icec, sst


c ... set program name for error messaging
      character*32 routine_name
      parameter ( routine_name = 'db_mod07_get_data' )

c ... external functions

      integer db_get_swath_metadata,db_read_flat_file,db_get_ancillary,
     &        db_read_mask_file
      external db_get_swath_metadata,db_read_flat_file,db_get_ancillary,
     &        db_read_mask_file

c ... Save Statements
      SAVE / mod07_data /, / mod07_debug /

c ... Data Statements

c ... Initialize 1km radiance storage array with the bad value

      do k = 1, nbands
        do j = 1, max_line
          do i = 1, max_pixel
            radiance1( i, j, k ) = bad_value
          end do
        end do
      end do

c ... Extract the scan type indicating day or night mode from L1B file

      scan_flag = -1
      mirror_side = -1
      status = db_get_swath_metadata(scan_1km_lun,scan,scan_flag,
     &                               mirror_side)
      if (status .ne. 0) then
          call message(routine_name,
     &                'FAILED - Unable to extract scan metadata. ',
     &                 0, 2)
      endif


c ... Now extract the radiance information
      do k = 1, nbands

c ...   Initialize buffer arrays

        m = 1 
        do j = 1, max_line
          do i = 1, max_pixel
            buffer( m ) = bad_value
            v_flag( m ) = -1
            m = m + 1
          end do
        end do

c ...   Get single band of radiance data

        resol = 1
        cal_type = 'rad'

        write(req_bandname,*) bands(k)

        status = db_read_flat_file(scan,resol,req_bandname,cal_type,
     &                             l1b_1km_lun,datatype_1km,
     &                             interleave_1km,resolution_1km,
     &                             error_1km,offset_1km,samples_1km,
     &                             lines_1km,bands_1km,bandnames_1km,
     &                             bandunits_1km,max_pixel,max_line,
     &                             buffer,v_flag,data_size(1),
     &                             data_size(2))
        if (status .lt. 0) then
          if ( debug .gt. 0 ) then
            write(h_output, '(''Error reading L1B radiance'')' )
            write(h_output, '(5x,''read status = '',I9)') status
          endif
          write(band_no,'(I6)' ) bands(k)
          call message( routine_name,
     &      'Failed to extract 1km L1B data. for band ' //band_no
     &      ,0, 2 )
        endif

c ...   Copy good data into 1km radiance storage array

        n = 1
        do j = 1, data_size( 2 )
          do i = 1, data_size( 1 )
            if ( v_flag( n ) .eq. 0 .and. buffer( n ) .gt. 0.0 )
     &        radiance1( i, j, k ) = buffer( n )
            n = n + 1
          end do
        end do

      end do

c ... Get geolocation data from file

c ... Extract latitude information
 
      m = 1
      do j = 1, max_line
        do i = 1, max_pixel
          lat1( i, j ) = bad_value
          buffer( m ) = bad_value
          v_flag( m ) = -1
          m = m + 1
        end do
      end do
 
      resol = 1
      req_bandname = 'Latitude'
      cal_type = 'deg'

      status = db_read_flat_file(scan,resol,req_bandname,cal_type,
     &                           geo_1km_lun,datatype_geo,
     &                           interleave_geo,resolution_geo,
     &                           error_geo,offset_geo,samples_geo,
     &                           lines_geo,bands_geo,bandnames_geo,
     &                           bandunits_geo,max_pixel,max_line,
     &                           buffer,v_flag,data_size(1),
     &                           data_size(2))
      if (status .lt. 0) then
        if ( debug .gt. 0 ) then
          write(h_output, '(''Error reading Latitude. '')' )
          write(h_output, '(5x,''read status = '',I9)') status
        endif
        call message( routine_name,
     &    'Failed to extract Latitude information ' 
     &     ,0, 2 )
      endif

c ... Fill array
      n = 1
      do j = 1, data_size( 2 )
        do i = 1, data_size( 1 )
          if ( buffer( n ) .ge. -90.0 .and.
     &      buffer( n ) .le. 90.0 ) lat1( i, j ) = buffer( n )
            n = n + 1
        end do
      end do

c ... Extract longitude information
 
      m = 1
      do j = 1, max_line
        do i = 1, max_pixel
          lon1( i, j ) = bad_value
          buffer( m ) = bad_value
          v_flag( m ) = -1
          m = m + 1
        end do
      end do
 
      resol = 1
      req_bandname = 'Longitude'
      cal_type = 'deg'

      status = db_read_flat_file(scan,resol,req_bandname,cal_type,
     &                           geo_1km_lun,datatype_geo,
     &                           interleave_geo,resolution_geo,
     &                           error_geo,offset_geo,samples_geo,
     &                           lines_geo,bands_geo,bandnames_geo,
     &                           bandunits_geo,max_pixel,max_line,
     &                           buffer,v_flag,data_size(1),
     &                           data_size(2))
      if (status .lt. 0) then
        if ( debug .gt. 0 ) then
          write(h_output, '(''Error reading Longitude. '')' )
          write(h_output, '(5x,''read status = '',I9)') status
        endif
        call message( routine_name,
     &    'Failed to extract Longitude information '
     &     ,0, 2 )
       endif

c ... Fill array
      n = 1
      do j = 1, data_size( 2 )
        do i = 1, data_size( 1 )
          if ( buffer( n ) .ge. -180.0 .and.
     &      buffer( n ) .le. 180.0 ) lon1( i, j ) = buffer( n )
            n = n + 1
        end do
      end do


c ... Extract viewing zenith information
 
      m = 1
      do j = 1, max_line
        do i = 1, max_pixel
          zen1( i, j ) = bad_value
          buffer( m ) = bad_value
          v_flag( m ) = -1
          m = m + 1
        end do
      end do
 
      resol = 1
      req_bandname = 'SensorZenith'
      cal_type = 'deg'

      status = db_read_flat_file(scan,resol,req_bandname,cal_type,
     &                           geo_1km_lun,datatype_geo,
     &                           interleave_geo,resolution_geo,
     &                           error_geo,offset_geo,samples_geo,
     &                           lines_geo,bands_geo,bandnames_geo,
     &                           bandunits_geo,max_pixel,max_line,
     &                           buffer,v_flag,data_size(1),
     &                           data_size(2))
      if (status .lt. 0) then
        if ( debug .gt. 0 ) then
          write(h_output, '(''Error reading Viewing Zenith. '')' )
          write(h_output, '(5x,''read status = '',I9)') status
        endif
        call message( routine_name,
     &    'Failed to extract Viewing Zenith information ' 
     &     ,0, 2 )
       endif

c ... Fill array
      n = 1
      do j = 1, data_size( 2 )
        do i = 1, data_size( 1 )
          if (abs(buffer(n)) .lt. 67.0 .and. abs(buffer(n)).gt. 0.0)
     &      zen1( i, j ) = buffer( n )
            n = n + 1
        end do
      end do


c ... Extract surface elevation information
 
      m = 1
      do j = 1, max_line
        do i = 1, max_pixel
          elv1( i, j ) = bad_value
          buffer( m ) = bad_value
          m = m + 1
        end do
      end do
 
      resol = 1
      req_bandname = 'Elevation'
      cal_type = 'm'

      status = db_read_flat_file(scan,resol,req_bandname,cal_type,
     &                           geo_1km_lun,datatype_geo,
     &                           interleave_geo,resolution_geo,
     &                           error_geo,offset_geo,samples_geo,
     &                           lines_geo,bands_geo,bandnames_geo,
     &                           bandunits_geo,max_pixel,max_line,
     &                           buffer,v_flag,data_size(1),
     &                           data_size(2))
      if (status .lt. 0) then
        if ( debug .gt. 0 ) then
          write(h_output, '(''Error reading Surface Elevation. '')' )
          write(h_output, '(5x,''read status = '',I9)') status
        endif
        call message( routine_name,
     &    'Failed to extract Surface Elevation information ' 
     &     ,0, 2 )
       endif

c ... Fill array
c ...  (The Dead Sea is the lowest place on the Earth's surface
c ...  at about 400 meters (1,300 feet) below sea level.
c ...  Mt. Everest is the highest at about 8840 meters (29,000 ft).)

      n = 1
      do j = 1, data_size( 2 )
        do i = 1, data_size( 1 )
          if ( buffer( n ) .ge. -400.0 .and.
     &      buffer( n ) .le. 8840.0 ) elv1( i, j ) = buffer( n )
            n = n + 1
        end do
      end do

c ... Extract land/sea mask information
 
      m = 1
      do j = 1, max_line
        do i = 1, max_pixel
          land1( i, j ) = -1
          buffer( m ) = bad_value
          m = m + 1
        end do
      end do
 
      resol = 1
      req_bandname = 'LandSea'
      cal_type = 'flg'

      status = db_read_flat_file(scan,resol,req_bandname,cal_type,
     &                           geo_1km_lun,datatype_geo,
     &                           interleave_geo,resolution_geo,
     &                           error_geo,offset_geo,samples_geo,
     &                           lines_geo,bands_geo,bandnames_geo,
     &                           bandunits_geo,max_pixel,max_line,
     &                           buffer,v_flag,data_size(1),
     &                           data_size(2))
      if (status .lt. 0) then
        if ( debug .gt. 0 ) then
          write(h_output, '(''Error reading Land Sea Mask. '')' )
          write(h_output, '(5x,''read status = '',I9)') status
        endif
        call message( routine_name,
     &    'Failed to extract Land Sea Mask information ' 
     &     ,0, 2 )
       endif

c ... Fill array
      n = 1
      do j = 1, data_size( 2 )
        do i = 1, data_size( 1 )
          if ( buffer( n ) .ge. 0.0 .and.
     &      buffer( n ) .le. 10.0 ) land1( i, j ) =
     &      int( buffer( n ) )
            n = n + 1
        end do
      end do

c-----------------------------------------------------------------------
c     READ NON-MODIS ANCILLARY DATA
c-----------------------------------------------------------------------

c ... Get ancillary data for 1km bands

      lun_icec = 0
      lun_ozone = 0
      lun_sst = 0
      lun_nisen = 0
      lun_nises = 0

      do j = 1, max_line
        do i = 1, max_pixel

c ...     Set default output values

          tem1( i, j ) = bad_value
          wat1( i, j ) = bad_value
          pre1( i, j ) = bad_value
          pwat1( i, j ) = bad_value

c ...     If lat/lon is in range, get ancillary data

          if ( lat1( i, j ) .ge.  -90.0 .and.
     &         lat1( i, j ) .le.   90.0 .and.
     &         lon1( i, j ) .ge. -180.0 .and.
     &         lon1( i, j ) .le.  180.0 ) then

              status=db_get_ancillary(anc_met_lun,lun_ozone,lun_icec,
     &               lun_sst,lun_nisen,lun_nises,lat1(i,j),lon1(i,j),
     &               pres,temp,mixr,land,sfctmp,sfcpres,pwat,ugrd,vgrd,
     &               ozone,icec,sst,nise) 
              if (status .lt. 0) then 
                  call message( 'routine_name',
     &                          'Error reading ancillary files.',
     &                           0, 2 )
               endif

c ...         Get interpolated surface pressure and TPW for this lat/lon

               call db_get_sfcpres_pwat(anc_met_lun,lat1(i,j), 
     &                              lon1(i,j),sfcpres,pwat)

c ...          If data values are good, save input values of
c ...          surface temperature, water vapor, and pressure
c ...          (note: only surface pressure is used in retrieval)

               if ( sfctmp .ge. 0.0 )    tem1( i, j ) = sfctmp
               if ( mixr( 1 ) .ge. 0.0 ) wat1( i, j ) = mixr( 1 )
               if ( sfcpres .ge. 0.0 )   pre1( i, j ) = sfcpres
               if ( pwat .ge. 0.0 )   pwat1( i, j ) = pwat

          endif

        end do

      end do

c ... Get cloud mask result array 

      m = 1
      do k = 1, max_line
        do j = 1, max_pixel
          mask1( j, k ) = 0
          m_buf( m ) = 0
          m = m + 1
        end do
      end do

      req_bandname = 'byte 1'
      resol = 1
      cal_type = ' '
      error_mask = 0.0
      do n = 1 , NMASK
         bunits_mask(n) = '  '
      enddo
      status = db_read_mask_file(scan,resol,req_bandname,cal_type,
     &                           mask_lun,datat_mask,interl_mask,
     &                           resol_mask,error_mask,
     &                           offset_mask,sampl_mask,
     &                           lines_mask,bands_mask,bnames_mask,
     &                           bunits_mask,max_pixel,max_line,
     &                           m_buf,v_flag,data_size(1),
     &                           data_size(2))
      if (status .lt. 0) then
        if ( debug .gt. 0 ) then
          write(h_output, '(''Error reading Cloud Mask Byte 1. '')' )
          write(h_output, '(5x,''read status = '',I9)') status
        endif
        call message( routine_name,
     &    'Failed to extract Cloud Mask information '
     &     ,0, 2 )
       endif

c ... Fill array
      n = 1
      do j = 1, data_size( 2 )
        do i = 1, data_size( 1 )
          mask1( i, j ) = m_buf( n )
          n = n + 1
        end do
      end do

c ...............................................................
c ... Write debug information ...................................

      if ( debug .gt. 0 ) then

        write( h_output, '(/,a,/)' ) 'MOD07_GET_DATA DEBUG INFO'

        i = 676
        j = 6
        write( h_output, '( ''Data for Nadir Pixel at '', ' //
     &    ' ''Scan'',i4,'', Pixel'',i4,'', Line'',i2)' ) scan, i, j

        mask_byte = int( mask1( i, j ) )
        write( h_output,
     &    ' (''Lat ='',f9.3,'', Lon ='',f10.3, ' //
     &    ' '', Elev = '',f7.1, '', Landmask ='',i3, ' //
     &    ' '', Cloudmask Bit 0 ='',i2,'', Cloudmask Bits 1:2 ='',i3)' )
     &    lat1( i, j ), lon1( i, j ), elv1( i, j ), land1( i, j ),
     &    iand( mask_byte, 1 ),
     &    iand( mask_byte, 6 )

        write( h_output, '(''Ancillary Data: '' ' //
     &    ' '', Sfc Temp ='',f7.2,'', K, Mixing Ratio ='',f7.2, ' //
     &    ' '' g/kg, Sfc Pressure ='',f7.2,'' hPa'')' )
     &    tem1( i, j ), wat1( i, j ), pre1( i, j )

        write( h_output, '(''IR Bands: '',20i8)' )
     &    ( bands( k ), k = 1, nbands )

        write( h_output, '(''Radiance: '',20f8.2)' )
     &    ( radiance1( i, j, k ), k = 1, nbands )

      endif

      end
