C--------------------------------------------------------------------
C  Copyright (C) 2002,  Space Science and Engineering Center, 
C  University C  of Wisconsin-Madison, Madison WI.
C      
C  This program is free software; you can redistribute it 
C  and/or modify it under the terms of the GNU General 
C  Public License as published by the Free Software Foundation; 
C  either version 2 of the License, or (at your option) any 
C  later version.
C
C  This program is distributed in the hope that it will be 
C  useful, but WITHOUT ANY WARRANTY; without even the implied 
C  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
C  See the  GNU General Public License for more details.
C
C  You should have received a copy of the GNU General Public 
C  License along with this program; if not, write to the Free 
C  Software Foundation, Inc., 59 Temple Place, Suite 330, 
C  Boston, MA  02111-1307 USA
C--------------------------------------------------------------------
C
C
      subroutine db_mod07_write_products(out_handle,hdr_lun,scan,nscans,
     &                                   nboxes)

c-----------------------------------------------------------------------
c
c!F77
c
c!Description:
c    Write MOD07 retrieval products to the output file.
c
c!Input Parameters:
c    OUT_HANDLE    LUN for MOD07 output file
c    HDR_LUN       LUN for product header output file
c    SCAN          Scan number within L1B granule
c    NSCANS        Total number of scans in this data set
c    NBOXES        Number of retrieval boxes along the scan
c                  (e.g. 1354 pixels, 5x5 sampling gives 1354/5 boxes)
c
c!Output Parameters:
c    None
c
c     The following arrays in COMMON /MOD07_DATA/ are written to output:
c     BRIGHTNESS_TEMP       Brightness temperature
c     RETR_TEMP_PROFILE     Retrieved temperature profile
c     RETR_DEWP_PROFILE     Retrieved dewpoint profile
c     RETR_HITE_PROFILE     Retrieved geopotential height profile
c     SFC_TEMP              Surface temperature
c     SFC_PRES              Surface pressure
c     SFC_ELEV              Surface elevation
c     WATER_VAPOR           Water vapor for entire column, 
c                           by integrating regression profiles
c     WATER_VAPOR_DIRECT    Water vapor for entire column, by direct regression
c     WATER_VAPOR_LOW       Water vapor for surface to 900 hPa
c     WATER_VAPOR_HIGH      Water vapor for 700 hPa to 300 hPa
c     TOTAL_OZONE           Total column ozone
c     TOTAL_TOTALS          Total totals index
c     LIFTED_INDEX          Lifted index
c     K_INDEX               K index
c
c!Team-unique Header:
c     Developed by the MODIS Group, CIMSS/SSEC, UW-Madison.
c
c!End
c
c-----------------------------------------------------------------------
      
      implicit none


      include 'db_mod07uw_data.inc'
      
c ... arguments

      integer out_handle, scan, nscans, nboxes, hdr_lun

c ... local variables

      integer i ,  ii

      save / mod07_data / 

c ... local arrays
      character*2 band_no
      character*4 lev
      character*3 b_unit
      character*4 plevel(outlevels)

c ... External functions
      integer string_length
      external string_length

c ... data statments
      data plevel /  '5', '10',  '20',  '30',  '50',  '70',  '100',
     &             '150', '200', '250', '300', '400', '500', '620',
     &             '700', '780', '850', '920', '950', '1000' /

c ... Write each product array for this scan to output file

      b_unit = 'K'
      do i = 1 , nbands
        write(band_no,'(I2)') bands(i)
        call write_output( out_handle, hdr_lun, scan, nscans, nboxes, i,
     &    brightness_temp(1,1,i), 'Brightness_Temperature_B'// band_no ,
     &    b_unit )
      end do

      ii = 1 + nbands
      b_unit = 'K'
      call write_output( out_handle, hdr_lun, scan, nscans, nboxes, ii,
     &  sfc_temp, 'Surface_Temperature' , b_unit )

      ii = ii + 1
      b_unit = 'hPa'
      call write_output( out_handle, hdr_lun, scan, nscans, nboxes, ii,
     &  sfc_pres, 'Surface_Pressure' , b_unit )

      ii = ii + 1
      b_unit = 'm'
      call write_output( out_handle, hdr_lun, scan, nscans, nboxes, ii,
     &  sfc_elev, 'Surface_Elevation' , b_unit )

      b_unit = 'K'
      do i = 1, outlevels
        ii = ii + 1
        write(lev,'(a)') plevel(i)
        call write_output(out_handle, hdr_lun, scan, nscans, nboxes, ii,
     &    retr_temp_profile(1,1,i), 'Retrieved_Temperature_Profile_Lev'
     &    // lev(1:string_length(lev)) , b_unit )
      end do

      b_unit = 'K'
      do i = 1, outlevels
        ii = ii + 1
        write(lev,'(a)') plevel(i)
        call write_output(out_handle, hdr_lun, scan, nscans, nboxes, ii,
     &    retr_dewp_profile(1,1,i), 'Retrieved_Moisture_Profile_Lev'   
     &    // lev(1:string_length(lev)) , b_unit )
      end do

      b_unit = 'm'
      do i = 1, outlevels
        ii = ii + 1
        write(lev,'(a)') plevel(i)
        call write_output(out_handle, hdr_lun, scan, nscans, nboxes, ii,
     &    retr_hite_profile(1,1,i), 'Retrieved_Height_Profile_Lev'   
     &    // lev(1:string_length(lev)) , b_unit )
      end do

      ii = ii + 1
      b_unit = 'Dob'
      call write_output( out_handle, hdr_lun, scan, nscans, nboxes, ii,
     &  total_ozone, 'Total_Ozone' , b_unit )

      ii = ii + 1
      b_unit = 'K'
      call write_output( out_handle, hdr_lun, scan, nscans, nboxes, ii,
     &  total_totals, 'Total_Totals' , b_unit )

      ii = ii + 1
      b_unit = 'K'
      call write_output( out_handle, hdr_lun, scan, nscans, nboxes, ii,
     &  lifted_index, 'Lifted_Index' , b_unit )

      ii = ii + 1
      b_unit = 'K'
      call write_output( out_handle, hdr_lun, scan, nscans, nboxes, ii,
     &  k_index, 'K_Index' , b_unit )

      ii = ii + 1
      b_unit = 'cm'
      call write_output( out_handle, hdr_lun, scan, nscans, nboxes, ii,
     &  water_vapor, 'Water_Vapor' , b_unit )

      ii = ii + 1
      b_unit = 'cm'
      call write_output( out_handle, hdr_lun, scan, nscans, nboxes, ii,
     &  water_vapor_direct, 'Water_Vapor_Direct' , b_unit )

      ii = ii + 1
      b_unit = 'cm'
      call write_output( out_handle, hdr_lun, scan, nscans, nboxes, ii,
     &  water_vapor_low, 'Water_Vapor_Low' , b_unit )

      ii = ii + 1
      b_unit = 'cm'
      call write_output( out_handle, hdr_lun, scan, nscans, nboxes, ii,
     &  water_vapor_high, 'Water_Vapor_High' , b_unit )

      end

c-----------------------------------------------------------------------

      subroutine write_output(out_handle,hdr_lun,scan,nscans,nboxes,
     &                        req_band,array,arrnm,b_unit_name)
c-----------------------------------------------------------------------
c!F77
c
c!Description:
c    Write MOD07 retrieval products to the output file.
c
c!Input Parameters:
c    OUT_HANDLE    LUN for MOD06CT output file
c    HDR_LUN       LUN for product header output file
c    SCAN          MODIS scan number
c    NSCANS        Total number of scans in this data set
c    NBOXES        Number of 5x5 retrieval boxes for this scan
c    REQ_BAND      Output file band number
c    ARRAY         Array of product values
c                  (Bad values should be set to BAD_VALUE in the calling
c                  routine: see mod06uw_data.inc for BAD_VALUE value)
c    ARRNM         Name of the array (SDS name) in the output file
c
c!Output Parameters:
c    None
c
c!Team-unique Header:
c     Developed by the MODIS Group, CIMSS/SSEC, UW-Madison.
c
c!End
c-----------------------------------------------------------------------

      implicit none
      save

      
      include 'db_mod07uw_data.inc'
      
c ... arguments

      integer out_handle, scan, nboxes, nscans, req_band, hdr_lun
      real array(max_samp_pixel,max_samp_line)
      character*(*) arrnm, b_unit_name
      logical init

c ... local variables
            
      integer i, j, out, ii, out_samples, out_lines, rtn, fil_datatype,
     &        fil_lun, fil_resolution, fil_offset, fil_samples, 
     &        fil_lines, fil_bands, out_maxsamples, out_maxlines,
     &        req_lines, req_samples, hdr_flg
      

      real fill_float, fil_error

c ... Local arrays
      byte out_flag(max_samp_pixel*max_samp_line)
      character*4 fil_interleave
      character*72 text
      character*3 fil_bandunits(OUT_BAND)
      character*80 fil_bandnames(OUT_BAND)
      character*256 fil_desc
      real float_data( max_samp_pixel * max_samp_line )

c ... external routines
      external db_mod07_write_flat_file
      integer db_mod07_write_flat_file

      integer string_length
      external string_length

      data init /.true./, ii /0/

 
      if (init) then
c ...   Set output file parameters
        req_samples = nboxes
        req_lines = max_samp_line

        fil_lun = out_handle
        fil_datatype = 4
        fil_interleave = 'bil'
        fil_resolution = 5
        fil_desc = 'Direct Broadcast Atmospheric Profiles Product'
        fil_error = -327.68
        fil_offset = 0
        fil_samples = nboxes
        fil_lines = nscans * max_samp_line
        fil_bands = OUT_BAND
        out_maxsamples = max_samp_pixel
        out_maxlines = max_samp_line
        init = .false.
      endif

c ... Fill in band and band unit information
      ii = ii + 1
      if (ii .le. OUT_BAND) then
        fil_bandnames(ii) = arrnm
        fil_bandunits(ii) = b_unit_name(1:string_length(b_unit_name))
      endif

c ... If we have gathered all the band names, then write header
      if (ii .eq. OUT_BAND) then
          hdr_flg = 1
      else
          hdr_flg = 0
      endif

      out = 1
c ... initialize the validity flag holder
      do j = 1 , max_samp_line
        do i = 1 , nboxes
           out_flag(out) = 0
        enddo
        out = out + 1
      enddo

c ... Get the fill value
      
      fill_float = -327.68
      
c ... Create the scaled output array

      out = 1
      
      do j = 1, max_samp_line

        do i = 1, nboxes

c ...     Set default output value (fill value)
        
          float_data( out ) = fill_float

c ...     If array value is good, scale it and store it
        
          if ( abs( array( i, j ) - bad_value ) .gt. 1.0e-5 ) then

               float_data( out ) =  array( i, j ) 

          endif
          
          out = out + 1

        end do

      end do

c ... Write the output array and header to the output files

c ------ write the mask array to a flat output file
      rtn = db_mod07_write_flat_file(
     &                         scan,
     &                         req_samples,
     &                         req_lines,
     &                         req_band,
     &                         fil_lun,
     &                         hdr_flg,
     &                         hdr_lun,
     &                         fil_desc,
     &                         fil_datatype,
     &                         fil_interleave,
     &                         fil_resolution,
     &                         fil_error,
     &                         fil_offset,
     &                         fil_samples,
     &                         fil_lines,
     &                         fil_bands,
     &                         fil_bandnames,
     &                         fil_bandunits,
     &                         out_maxsamples,
     &                         out_maxlines,
     &                         float_data,
     &                         out_flag,
     &                         out_samples,
     &                         out_lines
     &                       )

      if ( rtn .ne. 0 ) then
        write(text, '(''Write failed on output array '',a)') arrnm
        call message( 'db_mod06_write_products', text ,
     &   0, 2 )
      endif
                
      end
c-----------------------------------------------------------------------
