C--------------------------------------------------------------------
C  Copyright (C) 2002,  Space Science and Engineering Center,
C  University C  of Wisconsin-Madison, Madison WI.
C
C  This program is free software; you can redistribute it
C  and/or modify it under the terms of the GNU General
C  Public License as published by the Free Software Foundation;
C  either version 2 of the License, or (at your option) any
C  later version.
C
C  This program is distributed in the hope that it will be
C  useful, but WITHOUT ANY WARRANTY; without even the implied
C  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
C  See the  GNU General Public License for more details.
C
C  You should have received a copy of the GNU General Public
C  License along with this program; if not, write to the Free
C  Software Foundation, Inc., 59 Temple Place, Suite 330,
C  Boston, MA  02111-1307 USA
C--------------------------------------------------------------------
C
C
      SUBROUTINE DB_MODIS_GES101(BRT, PSURF, PLAND, THETA, ALAT, AMONTH,
     &  TRC_LUN, ANG_LUN, PRES, TEMP, WVMR, TPW, TOZ, LSFC, ERRFLG, 
     &  ERRTXT, SWEMIS, TPW_RET, SKINT)

C-----------------------------------------------------------------------
C !F77
C
C !DESCRIPTION:
C     MODIS regression retrieval of atmospheric profiles of
C     temperature, moisture, and ozone at 101 levels. In addition,
C     surface skin temperature, microwave emissivity, and IR surface
C     emissivities and reflectivities are retrieved with 1km resolution.
C     Algorithm developed by Jun Li, CIMSS/SSEC.
C
C !INPUT PARAMETERS:
C     BRT      Brightness temperatures (K) for all 36 MODIS bands
C              (caller should set to 0.0 for bands 1-19 and 26).
C              Local array KUSE determines which bands are used.
C     PSURF    Surface pressure (hPa)
C     PLAND    Fraction of land (0.0-1.0)
C     THETA    Local view angle (aka senzor zenith angle, degrees)
C     ALAT     Latitude (degrees, -90S to +90N)
C     AMONTH   Month of year (1-12)
C     TRC_LUN  FORTRAN unit number for regression coefficient file
C              MODIS_REGCOEF_FAC3.0.bin (caller must open this file
C              in direct access mode with reclen=1276 bytes)
C     ANG_LUN  FORTRAN unit number for view angle file
C              MODIS_senzen.bin (caller must open this file
C              in direct access mode with reclen=2720 bytes)
C
C !OUTPUT PARAMETERS:
C     PRES     Pressure levels for retrieved profiles (hPa)
C     TEMP     Retrieved profile of atmospheric temperature (K)
C     WVMR     Retrieved profile of water vapor mixing ratio (g/kg)
C     TPW      Total precipitable water vapor (cm)
C     TOZ      Total ozone (Dobsons)
C     LSFC     Level index of surface in pressure profile
C     ERRFLG   Error flag (0=success, 1=failure)
C     ERRTXT   Error text
C     SWEMIS   Short wave emissivity used in retrieval
C     TPW_RET  Total precipitable water vapor (cm) retrieved directly
C              instead of integrated from a retrieved profile
C
C !REVISION HISTORY:
C     26 February 2002: SWS made updates for delivery:
C        - Now uses 11 predictors instead of 12.  Instead of individual
C          bands 24,25,27-36, predictors are brightness temperature 
C          difference between 25-24, and individual bands 27 through 36
C        - Included 7 BT classes: perform retrieval using only that portion
C          of the training data set with band 31 BT in the same class.
C          Classes are < 245, 245-269, 269-285, 285-294, 294-300, 300-310, > 310
C        - Included output parameters swemis and tpw_ret. 
C
C
C    May 2003: SWS updated
C        - Call to tprecw_new2.f instead of tprecw.f
C        - Added retrieved surface skin temperature variable skint
C        - Removed band 24 from the regression (kuse), and removed 
C          band 25-24 difference
C        - Updated version number to agree with new coefficients, 2003073
C
C !TEAM-UNIQUE HEADER:
C     Developed by the MODIS Group, CIMSS/SSEC, UW-Madison.
C
C !END
C-----------------------------------------------------------------------

      implicit none
      
c ... Include files
      include 'platform_name.inc'
      include 'hdf.inc'
            
c ... Parameters
      integer nl, nemis, nall
      parameter (nl = 101, nemis = 7, nall = 36)
      integer leng
      parameter (leng = 3 * nl + 2 + 2 * nemis)
      integer nxx,nx

C --------------------
C	Number of predictors, nxx 11
C --------------------
      parameter (nxx=11, nx = 2*nxx + 4)
      integer ntwx, ntwy
      parameter (ntwx = nx, ntwy = leng)
      integer numang, numcls

C Add 7 BT Classes: numcls 
      parameter (numang = 680,numcls=7)
      integer nx1, nrx
      parameter (nx1 = nx + 1, nrx = nx1 * numang * numcls)

c ... Array arguments
      real brt(nall), pres(nl), temp(nl), wvmr(nl)

c ... Scalar arguments
      real psurf, pland, theta, alat, amonth, tpw, toz
      integer trc_lun, ang_lun, lsfc, errflg
      character*(*) errtxt

c ... new variables for directly retrieved TPW
      real tpw_ret, skint

c ... Local scalars
      integer i, ii, ixx, j, l, lang, init, ios, reclen, recnum, kk
      real sum, mindiff, thisdiff, wmax
      logical opened

c ... Local arrays
      real angp(numang), coef(ntwx, ntwy), coeftw(nrx, ntwy),
     &  xtw(ntwx), ytw(ntwy), ytwb(ntwy), header(ntwy)
      real rtv(leng), ozon(nl), profile(nl)
      real xx
      integer kuse(nall)
      integer ncls
      real swemis

c ... HDF variables
      character*100 sds_name
      integer varid,rank,dimsizes(2),data_type,num_attrs,rtn
      integer start(2), stride(2), edge(2)

c ... External functions
      real satmix, sbflip
      logical big_endian
      external satmix, big_endian, sbflip
      integer sfselect, sfginfo, sfrdata, sfn2index, sfendacc
      external sfselect, sfginfo, sfrdata, sfn2index, sfendacc
      
c ... Save statement
      save
      
c ... Data statements

C------------------------------------------------------
c ... Band use flags (1=use band, 0=don't use band)
C------------------------------------------------------
      data kuse/23*0, 0, 1, 0, 1, 9*1/

c ... Initialization flag (1=initialize, 0=don't initialize)      
      data init/1/

c ... Pressure profile at 101 levels (hPa)
      data profile/0.005,.016,.038,.077,.137,.224,.345,.506,.714,
     &  .975,1.297,1.687,2.153,2.701,3.340,4.077,4.920,
     &  5.878,6.957,8.165,9.512,11.004,12.649,14.456,16.432,
     &  18.585,20.922,23.453,26.183,29.121,32.274,35.651,39.257,
     &  43.100,47.188,51.528,56.126,60.989,66.125,71.540,77.240,
     &  83.231,89.520,96.114,103.017,110.237,117.777,125.646,133.846,
     &  142.385,151.266,160.496,170.078,180.018,190.320,200.989,212.028,
     &  223.441,235.234,247.408,259.969,272.919,286.262,300.000,314.137,
     &  328.675,343.618,358.966,374.724,390.893,407.474,424.470,441.882,
     &  459.712,477.961,496.630,515.720,535.232,555.167,575.525,596.306,
     &  617.511,639.140,661.192,683.667,706.565,729.886,753.628,777.790,
     &  802.371,827.371,852.788,878.620,904.866,931.524,958.591,986.067,
     &  1013.948,1042.232,1070.917,1100.000/
     
c-----------------------------------------------------------------------
c     INITIALIZATION
c-----------------------------------------------------------------------

c ... Intialize the output array
      do i = 1, leng
        rtv(i) = 0.0
      end do
      
c ... Set error flag and text
      errflg = 0
      errtxt = ' '
      
c ... Read view angle and regression coefficient files if required
      if (init .eq. 1) then

c ...   Check that view angle file was opened with
c ...   correct record length (2720 bytes)
        inquire(ang_lun, opened=opened, recl=reclen)
        if (.not. opened) then
          errflg = 1
          errtxt = 'View angle file is not open'
        endif
        if (reclen .ne. 2720) then
          errflg = 1
          errtxt = 'View angle file record length is incorrect'
        endif
        
c ...   Read view angle file
        recnum = 1
        read (ang_lun, rec=recnum, iostat=ios) angp
        if (ios .ne. 0) then
          errflg = 1
          errtxt = 'Error reading view angle file'
          return
        endif
c ...   Flip bytes if necessary
        if (.not. big_endian()) then
          do kk = 1 , numang
             angp(kk) = sbflip(angp(kk))
          enddo
        endif


c ...   Read the header record of the coefficient file
        recnum = 1
c ...   Get the variable id number
        varid  = sfselect(trc_lun, sfn2index(trc_lun, 'coefficients'))
	if (varid  .eq. -1) then
          errflg = 1
          errtxt = 'Error getting regression coefficient variable id'
          return
        endif
c ...   Get the number of pixels, lines, and scans
        rtn = sfginfo(varid,sds_name,rank,dimsizes,data_type,num_attrs)
        if (dimsizes(1) .ne. ntwy  .or. dimsizes(2) .ne. nrx+1) then
          errflg = 1
          errtxt = 'Error extracting correct dimensions from coefficient file '
          return
        endif

c ...   Set extraction parameters
	start(1) = 0
	start(2) = 0
c ...   Set stride vector
        stride(1) = 1
        stride(2) = 1
c ...   Set stride vector
        edge(1) = ntwy-1
        edge(2) = 1
        rtn = sfrdata(varid, start, stride, edge, header)

c ...   Read coefficient data (float values, no scaling)
        rtn = sfrdata(varid, start, stride, edge, header)
        if (rtn .eq. -1) then
          errflg = 1
          errtxt = 'Error reading regression coefficient file'
          return
        endif

c ...   Check the following items in the header record
c ...   (1) Satellite number (1=Terra, 2=Aqua)
c ...   (2) Creation date (YYYYDDD, Current version=2003073)
c ...   (NOTE: 'platform_name' is defined in platform_name.inc')
        if (nint(header(1)) .ne. 1 .and.
     &      nint(header(1)) .ne. 2) then
          errflg = 1
          errtxt = 'Invalid satellite number in regression coefficient file'
          return
        endif
        if ((platform_name(1:5) .eq. 'Terra' .and. nint(header(1)) .eq. 2) .or.
     &      (platform_name(1:4) .eq. 'Aqua'  .and. nint(header(1)) .eq. 1)) then
          errflg = 1
          errtxt = 'Mismatch between platform name and satellite number in regression coefficient file'
          return
        endif          
        if (nint(header(2)) .ne. 2003073) then
          errflg = 1
          errtxt = 'Invalid version number in regression coefficient file'
          return
        endif          
        
c ...   Read the regression coefficients
        do i = 1, nrx

          start(1) = 0
          start(2) = i
c ...     Read this record
          rtn = sfrdata(varid, start, stride, edge, ytw)
          if (rtn .eq. -1) then
            errflg = 1
            errtxt = 'Error reading regression coefficient file'
            return
          endif

c ...     Save the coefficients          
          do j = 1, ntwy
            coeftw(i, j) = ytw(j)
          end do
          
        end do

c ...   Unset the initialization flag
        init = 0
c ...   End access to the HDF coefficient file
        rtn = sfendacc(varid)

      end if


c-----------------------------------------------------------------------
c     RETRIEVAL
c-----------------------------------------------------------------------

c ... Get coefficient index for this view angle
      lang = 1
      mindiff = abs(angp(1) - theta)
      do i = 2, numang
        thisdiff = abs(angp(i) - theta)
        if (thisdiff .lt. mindiff) then
          mindiff = thisdiff
          lang = i
        endif
      end do

C Add 7 BT Classes
c ... Get the class number: ncls

        xx = brt(31) 

        if(xx .le. 245.0) ncls = 1
        if(xx .gt. 245.0 .and. xx .le. 269.0) ncls = 2
        if(xx .gt. 269.0 .and. xx .le. 285.0) ncls = 3
        if(xx .gt. 285.0 .and. xx .le. 294.0) ncls = 4
        if(xx .gt. 294.0 .and. xx .le. 300.0) ncls = 5
        if(xx .gt. 300.0 .and. xx .le. 310.0) ncls = 6
        if(xx .gt. 310.0) ncls = 7

C ----------------------------------------------------------

c ... Get coefficients for this view angle
      do i = 1,ntwx

C Add BT Classes (12 nov 01)
c        ii = (lang - 1) * nx1 + i
        ii = (ncls-1)*numang*nx1 + (lang - 1) * nx1 + i
        do j = 1, ntwy
          coef(i, j) = coeftw(ii, j)
        end do
      end do

C Add BT Classes (12 nov 01)
c      ii = lang * nx1
      ii = (ncls-1)*numang*nx1 + lang * nx1

      do j = 1, ntwy
        ytwb(j) = coeftw(ii, j)
      end do

C Removed predictor BT25 - BT24 difference because we're not using band 24
C and we improved emissivity in training data

c ... linear term of regression equation
      ixx=0

      do l = 1, nall
        if (kuse(l) . ne . 0) then
          ixx = ixx + 1
          xtw(ixx) = brt(l)
        end if
      end do
  
      do l = 1, nall
        if (kuse(l) . ne . 0) then
          ixx = ixx + 1
          xtw(ixx) = brt(l)**2 / 250.0
        end if
      end do

c ... Additional predictors

c ... Surface pressure
      ixx = ixx + 1
      xtw(ixx) = psurf

c ... Percentage of land
      ixx = ixx + 1
      xtw(ixx) = pland

c ... Latitude
      ixx = ixx + 1
      xtw(ixx) = alat

c ... Month
      ixx = ixx + 1
      xtw(ixx) = amonth

c ... Evaluate regression equation
      do i = 1, ntwy
      
        sum = 0.0
        do j = 1, ntwx
          sum = sum + xtw(j) * coef(j, i)
        end do
        rtv(i) = sum + ytwb(i)
        
c ...   Water vapor mixing ratios and ozone partial pressures
c ...   are retrieved as logarithms, so convert to actual values
        if (i .le. 3 * nl .and. i .gt. nl) rtv(i) = exp(rtv(i))
        
c ...   22 jan 02: also take the exponent of retrieved tpw 
c ...   tpw because we saved this as a logarithm in the coefficients
        if (i .eq. nl*3+2) rtv(i) = exp(rtv(i))

c ...   Limit minimum retrieved value for any parameter to 0.001        
        if (rtv(i) .le. 0.001) rtv(i) = 0.001
        
      end do

c ... Contents of array RTV (319 elements) are as follows:
c ...   Elements   1-101 are atmospheric temperatures (K)
c ...   Elements 102-202 are water vapor mixing ratios (g/kg)
c ...   Elements 203-303 are ozone partial pressures (ppmv)
c ...   Element  304     is surface skin temperature (K)

c ...   directly retrieved tpw has replaced mwemis:
c ...   Element  305    WAS 50.3 GHz surface emissivity
c ...   Element  305    IS NOW integrated TPW (cm)

c ...   Elements 306-312 are IR surface emissivities
c ...   Elements 313-319 are IR surface reflectivities

c ... Extract retrieved profiles
      do i = 1, nl
        pres(i) = profile(i)
        temp(i) = rtv(i)
        wvmr(i) = rtv(i + nl)
        ozon(i) = rtv(i + (nl * 2))
      end do

C 9 oct 01: save swemis (element 310 (4 micron), nl*3 + 7)
      swemis = rtv(nl*3 + 7)

C 17jan02: save directly retrieved tpw (element 305, nl*3 + 2)
      tpw_ret = rtv(nl*3 + 2)

C 1oct02: save skin temperature (element 304, nl*3 + 1)
      skint = rtv(nl*3 + 1)

c ... Do not allow retrieved water vapor mixing ratios greater than
c ... saturation mixing ratio
      do i = 1, nl
        wmax = satmix(pres(i), temp(i))
        wvmr(i) = min(wvmr(i), wmax) 
      end do
            
c-----------------------------------------------------------------------
c     DERIVED PRODUCTS
c-----------------------------------------------------------------------

c ... Find surface level in pressure profile
      lsfc = 1
      do i = 1, nl - 1
        if (psurf .ge. (pres(i) + 5.0)) lsfc = i + 1
      end do
      
c ... Compute total precipitable water (centimeters)
      call tprecw_new2(pres, wvmr, tpw, nl, lsfc, psurf)
      
c ... Compute total ozone (Dobson units)
      call total_ozone(pres, ozon, nl, lsfc, toz)

      END
