C--------------------------------------------------------------------
C  Copyright (C) 2002,  Space Science and Engineering Center, 
C  University C  of Wisconsin-Madison, Madison WI.
C      
C  This program is free software; you can redistribute it 
C  and/or modify it under the terms of the GNU General 
C  Public License as published by the Free Software Foundation; 
C  either version 2 of the License, or (at your option) any 
C  later version.
C
C  This program is distributed in the hope that it will be 
C  useful, but WITHOUT ANY WARRANTY; without even the implied 
C  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
C  See the  GNU General Public License for more details.
C
C  You should have received a copy of the GNU General Public 
C  License along with this program; if not, write to the Free 
C  Software Foundation, Inc., 59 Temple Place, Suite 330, 
C  Boston, MA  02111-1307 USA
C--------------------------------------------------------------------
C
C
      Program profiles
      implicit none

c-----------------------------------------------------------------------
c
c!F77
c
c!Description:
c    This is the main program for the Direct Broadcast
c    version of the Atmospheric Profiles product (MODIS product MOD07).
c    This products includes retrievals of temperature and moisture,
c    water vapor, total column ozone and stability indices.
c
c!Input Parameters:
c    To run:  profiles.exe profiles.cfg data_month
c
c The program reads a list of input files from the profiles.cfg
c  file.  
c   
c  Required 
c   Inputs Are:  MODIS 1 km flat file radiances and reflectances
c                MODIS flat file geolocation parameters
c                MODIS flat file Metadata information (Day/Night
c                       flag, etc.)
c                Global Data Assimilation System (GDAS) model
c                       profiles of temperature and moisture
c                Regression Coefficient Files
c                       (located in ../static directory)
c                Viewing Angle Correction File
c                       (located in ../static directory)
c                Cloud Mask binary flat file
c   
c                month - Data set numeric month (1-12).  Used to 
c                        determine correct coefficients
c!Output Parameters:
c
c  Ouput:  Writes binary flat file of float arrays of 83 output
c          parameters.  For more specific
c          information on the arrays included in the output file,
c          see the Atmospheric_Profiles_Description.txt file.
c
c!References and Credits:
c          See ATBD-MOD-07. 
c 
c!End
c
c-----------------------------------------------------------------------

C#######################################################################
C     DECLARATIONS
C#######################################################################

c ... UW include files

      include 'db_mod07uw_data.inc'
      include 'db_mod07uw_debug.inc'
      include 'platform_name.inc'


c ... output scalars

      real error_1km, error_geo

c ... Local arrays

      character*4 interleave_1km,interleave_geo,interl_mask,cmonth
      character*5 sat_name
      character*80 bandnames_1km(INBAND),bandnames_geo(20),
     &             bnames_mask(NMASK),
     &             bandunits_1km(INBAND),bandunits_geo(20)
      character*132 cfgname
      
      character*(PATH_MAX) l1b_1km_hdr,l1b_geo_hdr,mask_hdr


c ... Local scalars

      real     landfrac, pcgood, pcbad
      integer  nscans, npixels, scan, num_arg, iargc, good_total,
     &         mod07_lun, ang_lun, trc_lun, ngood, iostat,
     &         ncloudy, nland, flag, month, bias_lun, det_lun,
     &         out_lines_5km, out_elements_5km, line, pixel,
     &         nmissing, beg_lin, nlins, beg_ele, neles, ibes, beg_scan,
     &         hdr_lun,l1b_1km_lun, geo_1km_lun, mask_lun, scan_1km_lun,
     &         anc_met_lun, datatype_1km,datatype_geo,resol_mask,
     &         resolution_1km,resolution_geo,offset_1km,offset_geo,
     &         offset_mask,samples_1km,samples_geo,sampl_mask,bands_1km,
     &         bands_geo,bands_mask,lines_1km,lines_geo,lines_mask,
     &         datat_mask

c ... Save Statements
      save /platform_name_common/
     
      save / mod07_data /, / mod07_debug / 

c ... External functions

      integer copy_info
      external copy_info
      
      
C#######################################################################
C     INITIALIZATION
C#######################################################################

c ... Set MODIS IR band numbers used by MOD07
c ... (Can't use a DATA statement because BANDS is in a COMMON block)

      bands( 1) = 24
      bands( 2) = 25
      bands( 3) = 27
      bands( 4) = 28
      bands( 5) = 29
      bands( 6) = 30
      bands( 7) = 31
      bands( 8) = 32
      bands( 9) = 33
      bands(10) = 34
      bands(11) = 35
      bands(12) = 36

      write( *, '(2x,''Initiated DB Atmospheric Profiles '',
     &                 ''Processing  '')' ) 

c --- Get input argument of configuration file containing source
c ---  input and output file information.

c --- Check number of arguments
      num_arg = iargc()
      if (num_arg .ne. 3) then
        print *, 'Usage: profiles.exe cfgname sat_name month'
        print *, 'where'
        print *, 'cfgname: name of the profiles configuration file'
        print *, 'sat_name: MODIS satellite platform name (Aqua or Terra)'
        print *, 'month: numeric month (1-12) of dataset'
        call exit(-1)
      endif

c ... Extract arguments
      call getarg(1, cfgname)
      call getarg(2, sat_name)
      call getarg(3, cmonth)
      read(cmonth,'(I2)',iostat=iostat) month
      
      if (month .lt. 1 .or. month .gt. 12) then 
        print *, 'Incorrect month entered: Must be (1-12) ',  month
        call exit(-1)
      endif

c ... Check input platform name to make sure it is either Aqua or Terra
      platform_name = ' '
      if (sat_name(1:5) .eq. 'Terra'   .or.
     +    sat_name(1:5) .eq. 'terra'   .or.
     +    sat_name(1:5) .eq. 'TERRA') then
          platform_name = 'Terra'
      elseif (sat_name(1:4) .eq. 'Aqua'   .or.
     +    sat_name(1:4) .eq. 'aqua'   .or.
     +    sat_name(1:4) .eq. 'AQUA') then
          platform_name = 'Aqua'
      else
          print '(''Error: Incorrect Satellite Platform name entered '', a)',
     +              sat_name(1: len(sat_name))
          call exit(-1)
      endif
c ... Write out platform name
      print '('' For Satellite Platform '', a)',
     +              platform_name(1: len(platform_name))


c ... Get debug value and processing interval information from PCF
      call db_mod07_get_rp(cfgname,debug,l1b_1km_hdr,l1b_geo_hdr,
     +                     mask_hdr,beg_lin,nlins,beg_ele,neles)

c ....................................................................

c *********************************************************************

c ... Open required files for this processing section
      call db_mod07_file_open(cfgname,neles,l1b_1km_lun,
     +                        geo_1km_lun,mask_lun,scan_1km_lun,
     +                        anc_met_lun,trc_lun,ang_lun,bias_lun,
     +                        det_lun,mod07_lun,hdr_lun,h_output)

c ... Get number of scans and pixels for 1km bands

      call db_mod07_Get_Metadata(l1b_1km_hdr,l1b_geo_hdr,mask_hdr,
     &     nscans,npixels,datatype_1km,datatype_geo,datat_mask,
     &     interleave_1km,interleave_geo,interl_mask,resolution_1km,
     &     resolution_geo,resol_mask,offset_1km,offset_geo,
     &     offset_mask,samples_1km,samples_geo,sampl_mask,lines_1km,
     &     lines_geo,lines_mask,error_1km,error_geo,bands_1km,
     &     bands_geo,bands_mask,bandnames_1km,bandnames_geo,
     &     bnames_mask,bandunits_1km,bandunits_geo)


c ... Check User defined processing intervals versus scan boundaries

      call db_mod07_chk_input(nscans,npixels,beg_lin,nlins,beg_ele,
     +                        neles,beg_scan,ibes,out_lines_5km,
     +                        out_elements_5km)

      good_total = 0

C#######################################################################
C     START SCIENCE PROCESSING
C#######################################################################

c ... Begin scan loop

      do scan = beg_scan , nscans

c ...   Write debug info

        write( *, '(''Processing scan # '',i4)' ) scan

c ...   Extract radiance, geolocation, mask, and ancillary data

        call db_mod07_get_data(l1b_1km_lun,geo_1km_lun,datatype_1km,
     &       datatype_geo,datat_mask,interleave_1km,interleave_geo,
     &       interl_mask,resolution_1km,resolution_geo,resol_mask,
     &       offset_1km,offset_geo,offset_mask,samples_1km,
     &       samples_geo,sampl_mask,lines_1km,lines_geo,lines_mask,
     &       error_1km,error_geo,bands_1km,bands_geo,bands_mask,
     &       bandnames_1km,bandnames_geo,bnames_mask,bandunits_1km,
     &       bandunits_geo,scan_1km_lun,anc_met_lun,mask_lun,
     &       scan)


c ...   Initialize output product arrays for this scan

        call mod07_initialize_output()

c ...   Start loop over pixel boxes (line and pixel is at corner of box)

        do line = 1 , ( max_line / isamp ) * isamp, isamp

          do pixel = ibes , ( npixels / isamp ) * isamp, isamp

c ...       Check this box for good data
            call db_mod07_check_box( line, pixel, ngood, nmissing, 
     &        ncloudy, nland )

c ...       If this box has enough good pixels, compute products
            if ( ngood .ge. min_ngood ) then

c ...         Compute land fraction
              landfrac = real(nland) / real(ngood)

c ...         Compute retrieval
              call db_mod07_compute_products(line,pixel,month,landfrac,
     &                           bias_lun,det_lun,trc_lun,ang_lun,flag)

c ...         Increment retrieval counter
              if ( flag .eq. 0 ) good_total = good_total + 1

            endif

          end do

        end do

c ...   Write products for this scan

c ...   write data to output file 
        call db_mod07_write_products(mod07_lun,hdr_lun,
     &       scan,nscans,out_elements_5km)

c ... End of loop over all scans

      end do

C#######################################################################
C     END SCIENCE PROCESSING
C#######################################################################

c ... Write out summary statistics to debug file
      pcgood = 100.*real(good_total)/
     &         real(out_lines_5km*out_elements_5km)
      pcbad = 100.-pcgood

      if (debug .gt. 0) then
        write(h_output, '(/,20x,''********************************'')')
        write(h_output, '(/,5x,''Total Number of Retrievals: '', i10)') 
     &                    good_total
        write(h_output, '(5x,''Percentage of good retrievals:'', f7.2)')
     &                    pcgood
        write(h_output, '(5x,''Percentage of no retievals '',
     &        ''(cloudy or bad data):'', f7.2)') pcbad
      endif

c ... close files for this section of code

      call db_mod07_file_close(l1b_1km_lun,geo_1km_lun,mask_lun,
     &                         scan_1km_lun,anc_met_lun,trc_lun,
     &                         ang_lun,mod07_lun,hdr_lun,bias_lun,
     &                         det_lun)

c ... Write debug info and then close debug file

      write( *, '(2x,''DB Atmospheric Profiles Processing'',
     &                 '' Completed. '')' ) 

      end
